<?php
/**
 * @version		1.5.4
 * @package		Joomla
 * @subpackage	Membership Pro
 * @author  Tuan Pham Ngoc
 * @copyright	Copyright (C) 2012 Ossolution Team
 * @license		GNU/GPL, see LICENSE.php
 */
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

/**
 * OS Membership Component Field Model
 *
 * @package		Joomla
 * @subpackage	Membership Pro
 * @since 1.5
 */
class OSMembershipModelField extends OSModel
{

	/**
	 * Store the custom fields
	 * @see OSModel::store()
	 */
	function store(&$data)
	{
		$row = & $this->getTable('osmembership', 'field');
		if ($data['id'])
		{
			$row->load($data['id']);
		}
		if ($row->is_core)
		{
			$ignores = array('name', 'field_type', 'fee_field');
		}
		else
		{
			$ignores = array();
		}
		if (!$row->bind($data, $ignores))
		{
			$this->setError($this->_db->getErrorMsg());
			return false;
		}
		if (!$row->check())
		{
			$this->setError($this->_db->getErrorMsg());
			return false;
		}
		if (!$row->id)
		{
			$row->ordering = $row->getNextOrder($this->getWhereNextOrdering());
		}
		
		if (!$row->store())
		{
			$this->setError($this->_db->getErrorMsg());
			return false;
		}
		$data['id'] = $row->id;
		
		return true;
	}

	/**
	 * Publish, unpublish the custom fields
	 * @see OSModel::publish()
	 */
	function publish($cid, $state)
	{
		$cids = implode(',', $cid);
		$sql = 'UPDATE #__osmembership_fields SET published=' . $state . ' WHERE id IN (' . $cids . ' ) AND name NOT IN("first_name", "email")';
		$this->_db->setQuery($sql);
		if ($this->_db->query())
			return true;
		else
			return false;
	}

	/**
	 * Delete the custom fields
	 * @see OSModel::delete()
	 */
	function delete($cid = array())
	{
		if (count($cid))
		{
			$db = JFactory::getDBO();
			$cids = implode(',', $cid);
			
			$sql = 'DELETE FROM #__osmembership_fields WHERE is_core = 0 AND id IN (' . $cids . ')';
			$db->setQuery($sql);
			$db->query();
			
			$sql = 'DELETE FROM #__osmembership_field_value WHERE field_id IN (' . $cids . ')';
			$db->setQuery($sql);
			$db->query();
		}
		
		return true;
	}
}