<?php
/**
* @version		1.5.4
* @package		Joomla
* @subpackage	OSMembership
* @author  Tuan Pham Ngoc
* @copyright	Copyright (C) 2012 Ossolution Team
* @license		GNU/GPL, see LICENSE.php
*/
// no direct access
defined('_JEXEC') or die();

/**
 * OS Membership controller
 * @package		Joomla
 * @subpackage	OSMembership
 * @since 1.5
 */
class OSMembershipController extends JControllerLegacy
{

	/**
	 * Constructor function
	 *
	 * @param array $config
	 */
	function __construct($config = array())
	{
		parent::__construct($config);
	}

	/**
	 * Display information
	 *
	 */
	function display()
	{
		$task = $this->getTask();
		$document = JFactory::getDocument();
		$styleUrl = JURI::base(true) . '/components/com_osmembership/assets/css/style.css';
		$document->addStylesheet($styleUrl, 'text/css', null, null);
		switch ($task)
		{
			case 'subscription_confirmation':
				JRequest::setVar('view', 'register');
				JRequest::setVar('layout', 'confirmation');
				break;
			default:
				$view = JRequest::getVar('view', '');
				if (!$view)
				{
					JRequest::setVar('view', 'plans');
					JRequest::setVar('layout', 'default');
				}
				break;
		}
				
		if (version_compare(JVERSION, '3.0', 'lt')) {
			OSMembershipHelper::loadBootstrap();
		}
				
		parent::display();
	}

	/**
	 *Perform renew membership 
	 */
	function renew_membership()
	{
		$renewOptionId = JRequest::getInt('renew_option_id');
		$fromSubscriptionId = JRequest::getInt('from_subscription_id', 0);
		$ret = OSMembershipHelper::canRenewMembership($renewOptionId, $fromSubscriptionId);
		if ($ret)
		{
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			if ($renewOptionId == OSM_DEFAULT_RENEW_OPTION_ID)
			{
				$query->select('b.id')
					->from('#__osmembership_subscribers AS a')
					->innerJoin('#__osmembership_plans AS b ON a.plan_id=b.id')
					->where('a.id=' . $fromSubscriptionId);
				$db->setQuery($query);
				$planId = (int) $db->loadResult();
				JRequest::setVar('id', $planId);
			}
			else
			{
				$query->select('plan_id')
					->from('#__osmembership_renewrates')
					->where('id=' . $renewOptionId);
				$db->setQuery($query);
				$rate = $db->loadObject();				
				JRequest::setVar('id', $rate->plan_id);
			}
			JRequest::setVar('view', 'register');
			JRequest::setVar('layout', 'default');
			$this->display();
		}
		else
		{
			JFactory::getApplication()->redirect('index.php', JText::_('OSM_INVALID_RENEW_MEMBERSHIP_OPTION'));
		}
	}

	function upgrade_membership()
	{
		$upgradeOptionId = JRequest::getInt('upgrade_option_id');
		$fromSubscriptionId = JRequest::getInt('from_subscription_id', 0);
		$ret = OSMembershipHelper::canUpgradeMembership($upgradeOptionId, $fromSubscriptionId);
		if ($ret)
		{
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			$query->select('to_plan_id')
				->from('#__osmembership_upgraderules')
				->where('id=' . $upgradeOptionId);
			$db->setQuery($query);
			$upgradeRule = $db->loadObject();
			//Set Plan ID
			JRequest::setVar('id', $upgradeRule->to_plan_id);
			JRequest::setVar('view', 'register');
			JRequest::setVar('layout', 'default');
			$this->display();
		}
		else
		{
			JFactory::getApplication()->redirect('index.php', JText::_('OSM_INVALID_RENEW_MEMBERSHIP_OPTION'));
		}
	}	
	/**
	 * Process subscription
	 */
	function process_subscription()
	{
		JRequest::checkToken() or jexit('Invalid Token');		
		$config = OSMembershipHelper::getConfig();
		if ($config->enable_captcha)
		{
			$session = JFactory::getSession();
			$securityCode = JRequest::getVar('security_code', '', 'post');
			if ($securityCode != $session->get('osm_security_code'))
			{
				JRequest::setVar('view', 'register');
				JRequest::setVar('layout', 'default');
				JRequest::setVar('captcha_invalid', 1);
				JRequest::setVar('id', JRequest::getInt('plan_id'));				
				$this->display();
				return;
			}
			$session->clear('osm_security_code');
		}		
		$post = JRequest::get('post', JREQUEST_ALLOWHTML);
		$model = $this->getModel('Register');
		$model->processSubscription($post);
	}

	function payment_confirm()
	{
		$model = $this->getModel('Register');
		$model->paymentConfirm();
	}

	function recurring_payment_confirm()
	{
		$model = $this->getModel('register');
		$model->recurringPaymentConfirm();
	}

	/**
	 * Show captcha image, using in the captcha form
	 */
	function show_captcha_image()
	{
		header("Cache-Control: no-cache, must-revalidate"); // HTTP/1.1
		header("Expires: Sat, 26 Jul 1997 05:00:00 GMT"); //
		while (@ob_end_clean());
		require_once JPATH_COMPONENT . '/helper/captcha.php';
		$captcha = new CaptchaSecurityImages();
		
		JFactory::getApplication()->close();
	}

	/**
	 * Download invoice
	 */
	function download_invoice()
	{
		$id = JRequest::getInt('id');
		OSMembershipHelper::downloadInvoice($id);
	}

	/**
	 * Process download a file
	 */
	function download_file()
	{
		$Itemid = JRequest::getInt('Itemid');
		$filePath = 'media/com_osmembership/upload';
		$fileName = JRequest::getVar('file_name', '');
		if (file_exists(JPATH_ROOT . '/' . $filePath . '/' . $fileName))
		{
			while (@ob_end_clean());
			OSMembershipHelper::processDownload(JPATH_ROOT . '/' . $filePath . '/' . $fileName, $fileName, true);
			exit();
		}
		else
		{
			$mainframe = JFactory::getApplication();
			$mainframe->redirect('index.php?option=com_osmembership&Itemid=' . $Itemid, JText::_('OSM_FILE_NOT_EXIST'));
		}
	}
		
	function calculate_coupon_discount() {
		$session = JFactory::getSession();				
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$config = OSMembershipHelper::getConfig();
		$planId = JRequest::getInt('plan_id', 0);
		$couponCode = JRequest::getVar('coupon_code', '', '');
		$amount = JRequest::getFloat('amount', 0, 'post');
		$trialAmount = JRequest::getFloat('trial_amount', 0, 'post');
		$regularAmount = JRequest::getFloat('regular_amount', 0, 'post');
		$nullDate = '0000-00-00';		
		$couponValid = 1 ;
		$discountAmount = 0;
		$trialDiscountAmount = 0 ;		
		$regularDiscountAmount = 0;
		$taxAmount = 0;
		$trialTaxAmount = 0;
		$regularTaxAmount = 0;
		if ($couponCode) {
			$query->select('*')
			->from('#__osmembership_coupons')
			->where('published=1')
			->where('code="' . $couponCode . '"')
			->where('(valid_from="' . $nullDate. '" OR DATE(valid_from) <= CURDATE())')
			->where('(valid_to="' . $nullDate. '" OR DATE(valid_to) >= CURDATE())')
			->where('(times = 0 OR times > used)')
			->where('(plan_id=0 OR plan_id=' . $planId . ')');
			$db->setQuery($query);			
			$coupon = $db->loadObject();			
			if ($coupon)
			{
				$session->set('osm_coupon_id', $coupon->id);
				if ($coupon->coupon_type == 0)
				{
					$discountAmount = $amount * $coupon->discount / 100;
					$trialDiscountAmount = $trialAmount * $coupon->discount / 100;
					$regularDiscountAmount = $regularAmount * $coupon->discount / 100;
				}
				else
				{
					$discountAmount = $coupon->discount;
					$trialDiscountAmount = $coupon->discount;
					$regularDiscountAmount = $coupon->discount;
				}
				if ($discountAmount > $amount)
					$discountAmount = $amount;
				if ($trialDiscountAmount > $trialAmount)
					$trialDiscountAmount = $trialAmount;
				if ($regularDiscountAmount > $regularAmount)
					$regularDiscountAmount = $regularAmount;
			}
			else
			{
				$couponValid = 0;
				$session->clear('osm_coupon_id');				
			}				
		}
		if ($config->enable_tax && $config->tax_rate > 0) {
			$taxAmount = $config->tax_rate*($amount - $discountAmount)/100;
			$trialTaxAmount = $config->tax_rate*($trialAmount - $trialDiscountAmount)/100;
			$regularTaxAmount = $config->tax_rate*($regularAmount - $regularDiscountAmount)/100;	
		}		
		$result = array(
			'coupon_valid' => $couponValid,
			'amount' => OSMembershipHelper::formatAmount($amount, $config),
			'discount_amount' => OSMembershipHelper::formatAmount($discountAmount, $config),
			'tax_amount' => OSMembershipHelper::formatAmount($taxAmount, $config),
			'gross_amount' => OSMembershipHelper::formatAmount($amount - $discountAmount + $taxAmount, $config),
			'trial_amount' => OSMembershipHelper::formatAmount($trialAmount, $config),
			'trial_discount_amount' => OSMembershipHelper::formatAmount($trialDiscountAmount, $config),
			'trial_tax_amount' => OSMembershipHelper::formatAmount($trialTaxAmount, $config),
			'trial_gross_amount' => OSMembershipHelper::formatAmount($trialAmount - $trialDiscountAmount + $trialTaxAmount, $config),
			'regular_amount' => OSMembershipHelper::formatAmount($regularAmount, $config),
			'regular_discount_amount' => OSMembershipHelper::formatAmount($regularDiscountAmount, $config),
			'regular_tax_amount' => OSMembershipHelper::formatAmount($regularTaxAmount, $config),
			'regular_gross_amount' => OSMembershipHelper::formatAmount($regularAmount - $regularDiscountAmount + $regularTaxAmount, $config)			
		);			
		echo json_encode($result);
		JFactory::getApplication()->close();
	}

	function validate_username() {
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);		
		$username = JRequest::getVar('username', '', 'post');
		$query->select('COUNT(*)')
		->from('#__users')
		->where('username="' . $username . '"');
		$db->setQuery($query);
		$total = $db->loadResult();
		if ($total)
			echo 0;
		else
			echo 1;				
		JFactory::getApplication()->close();
	}
	
	function validate_email() {
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$email = JRequest::getVar('email', '', 'post');
		$query->select('COUNT(*)')
		->from('#__users')
		->where('email="' . $email . '"');
		$db->setQuery($query);
		$total = $db->loadResult();
		if ($total)
		{
			echo 0;
		} else {
			echo 1;
		}		
		JFactory::getApplication()->close();
	}
}