<?php
/**
* @version		1.1.1
* @package		Joomla
* @subpackage	OS Membership
* @author  Tuan Pham Ngoc
* @copyright	Copyright (C) 2012 Ossolution Team
* @license		GNU/GPL, see LICENSE.php
*/
// no direct access
defined('_JEXEC') or die();

class OSMembershipHelper
{

	/**
	 * Get configuration data and store in config object
	 *
	 * @return object
	 */
	public static function getConfig()
	{
		static $config;
		if (!$config)
		{
			$db = JFactory::getDBO();
			$config = new stdClass();
			$sql = 'SELECT * FROM #__osmembership_configs';
			$db->setQuery($sql);
			$rows = $db->loadObjectList();
			for ($i = 0, $n = count($rows); $i < $n; $i++)
			{
				$row = $rows[$i];
				$key = $row->config_key;
				$value = stripslashes($row->config_value);
				$config->$key = $value;
			}
		}
		return $config;
	}

	/**
	 * Get specify config value
	 *
	 * @param string $key
	 */
	public static function getConfigValue($key)
	{
		$db = JFactory::getDBO();
		$sql = 'SELECT config_value FROM #__osmembership_configs WHERE config_key="' . $key . '"';
		$db->setQuery($sql);
		return $db->loadResult();
	}

	/**
	 * Load bootstrap lib
	 */
	public static function loadBootstrap($loadJs = true)
	{
		$app = JFactory::getApplication();
		$document = JFactory::getDocument();
		$config = self::getConfig();
		if ($loadJs)
		{
			$document->addScript(JUri::root() . 'components/com_osmembership/assets/bootstrap/js/jquery.min.js');
			$document->addScript(JUri::root() . 'components/com_osmembership/assets/bootstrap/js/jquery-noconflict.js');
			$document->addScript(JUri::root() . 'components/com_osmembership/assets/bootstrap/js/bootstrap.min.js');
		}
		if ($app->isAdmin() || $config->load_twitter_bootstrap_in_frontend !== '0')
			$document->addStyleSheet(JURI::root() . 'components/com_osmembership/assets/bootstrap/css/bootstrap.css');
	}

	/**
	 * Get Itemid of OS Membership Componnent
	 *
	 * @return int
	 */
	public static function getItemid()
	{
		$db = JFactory::getDBO();
		$user = JFactory::getUser();
		$sql = "SELECT id FROM #__menu WHERE link LIKE '%index.php?option=com_osmembership%' AND published=1 AND `access` IN (" . implode(',', $user->getAuthorisedViewLevels()) . ") ORDER BY `access`";
		$db->setQuery($sql);
		$itemId = $db->loadResult();
		if (!$itemId)
		{
			$Itemid = JRequest::getInt('Itemid');
			if ($Itemid == 1)
				$itemId = 999999;
			else
				$itemId = $Itemid;
		}
		return $itemId;
	}

	/**
	 * Check to see whether the ideal payment plugin installed and activated
	 * @return boolean
	 */
	public static function idealEnabled()
	{
		$db = JFactory::getDBO();
		$sql = 'SELECT COUNT(id) FROM #__osmembership_plugins WHERE name="os_ideal" AND published=1';
		$db->setQuery($sql);
		$total = $db->loadResult();
		if ($total)
		{
			require_once JPATH_ROOT . '/components/com_osmembership/plugins/ideal/ideal.class.php';
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get list of banks for ideal payment plugin
	 * @return array
	 */
	public static function getBankLists()
	{
		$idealPlugin = os_payments::loadPaymentMethod('os_ideal');
		$params = new JRegistry($idealPlugin->params);
		$partnerId = $params->get('partner_id');
		$ideal = new iDEAL_Payment($partnerId);
		$bankLists = $ideal->getBanks();
		return $bankLists;
	}

	/**
	 * Get country code
	 *
	 * @param string $countryName
	 * @return string
	 */
	public static function getCountryCode($countryName)
	{
		$db = JFactory::getDBO();
		$sql = 'SELECT country_2_code FROM #__osmembership_countries WHERE LOWER(name)="' . JString::strtolower($countryName) . '"';
		$db->setQuery($sql);
		$countryCode = $db->loadResult();
		if (!$countryCode)
			$countryCode = 'US';
		return $countryCode;
	}

	/**
	 * Load language from main component
	 *
	 */
	public static function loadLanguage()
	{
		static $loaded;
		if (!$loaded)
		{
			$lang = JFactory::getLanguage();
			$tag = $lang->getTag();
			if (!$tag)
				$tag = 'en-GB';
			$lang->load('com_osmembership', JPATH_ROOT, $tag);
			$loaded = true;
		}
	}

	/**
	 * Display copy right information
	 *
	 */
	public static function displayCopyRight()
	{
		echo '<div class="copyright" style="text-align:center;margin-top: 5px;"><a href="http://joomdonation.com/components/membership-pro.html" target="_blank"><strong>Membership Pro</strong></a> version 1.5.4, Copyright (C) 2010-2012 <a href="http://joomdonation.com" target="_blank"><strong>Ossolution Team</strong></a></div>';
	}

	/**
	 * Get active membership plans
	 */
	public static function getActiveMembershipPlans($userId = 0)
	{
		$activePlans = array();
		$activePlans[] = 0;
		if (!$userId)
		{
			$user = JFactory::getUser();
			$userId = $user->get('id');
		}
		if ($userId > 0)
		{
			$db = JFactory::getDBO();
			$sql = 'SELECT plan_id FROM #__osmembership_subscribers WHERE user_id=' . $userId . ' AND DATE(from_date) <= CURDATE() AND DATE(to_date) >= CURDATE() and published=1';
			$db->setQuery($sql);
			if (version_compare(JVERSION, '3.0', 'ge'))
				$activePlans = array_merge($activePlans, $db->loadColumn());
			else
				$activePlans = array_merge($activePlans, $db->loadResultArray());
		}
		
		return $activePlans;
	}

	/**
	 * Get total subscribers of a subscription plan based on status
	 * @param int $planId
	 * @param int $status
	 */
	public static function countSubscribers($planId, $status = -1)
	{
		$db = JFactory::getDBO();
		$where = array();
		$where[] = 'plan_id=' . $planId;
		if ($status != -1)
		{
			$where[] = 'published=' . $status;
		}
		$sql = 'SELECT COUNT(*) FROM #__osmembership_subscribers WHERE ' . implode(' AND ', $where);
		$db->setQuery($sql);
		return (int) $db->loadResult();
	}

	/**
	 * Check to see whether the current user can renew his membership using the given option
	 * @param int $renewOptionId
	 * @return boolean
	 */
	public static function canRenewMembership($renewOptionId, $fromSubscriptionId)
	{
		return true;
	}

	/**
	 * Check to see whether the current user can upgrade his membership using the upgraded option
	 * @param int $upgradeOptionId
	 * @return boolean
	 */
	public static function canUpgradeMembership($upgradeOptionId, $fromSubscriptionId)
	{
		return true;
	}

	/**
	 * Upgrade a membership
	 * @param Object $row
	 */
	public static function processUpgradeMembership($row)
	{
		$rowSubscription = JTable::getInstance('OsMembership', 'Subscriber');
		$rowSubscription->load($row->from_subscription_id);
		$rowSubscription->to_date = date('Y-m-d H:i:s');
		$rowSubscription->published = 2;
		$rowSubscription->store();
		//Trigger plugins
		JPluginHelper::importPlugin('osmembership');
		$dispatcher = JDispatcher::getInstance();
		$dispatcher->trigger('onMembershipExpire', array($rowSubscription));
	}

	/**
	 * Get next membership id for this subscriber
	 */
	public static function getMembershipId()
	{
		$db = JFactory::getDbo();
		$sql = 'SELECT MAX(membership_id) FROM #__osmembership_subscribers';
		$db->setQuery($sql);
		$membershipId = (int) $db->loadResult();
		if (!$membershipId)
		{
			$membershipId = (int) self::getConfigValue('membership_id_start_number');
			if (!$membershipId)
				$membershipId = 1000;
		}
		else
		{
			$membershipId++;
		}
		
		return $membershipId;
	}

	/**
	 * Get the invoice number for this subscription record
	 */
	public static function getInvoiceNumber()
	{
		$db = JFactory::getDbo();
		$sql = 'SELECT MAX(invoice_number) FROM #__osmembership_subscribers';
		$db->setQuery($sql);
		$invoiceNumber = (int) $db->loadResult();
		if (!$invoiceNumber)
		{
			$invoiceNumber = (int) self::getConfigValue('invoice_start_number');
			if (!$invoiceNumber)
				$membershipId = 1;
		}
		else
		{
			$invoiceNumber++;
		}
		
		return $invoiceNumber;
	}

	/**
	 * Format invoice number
	 * @param string $invoiceNumber
	 * @param Object $config
	 */
	public static function formatInvoiceNumber($invoiceNumber, $config)
	{
		return $config->invoice_prefix . str_pad($invoiceNumber, $config->invoice_number_length ? $config->invoice_number_length : 4, '0', STR_PAD_LEFT);
	}

	/**
	 * Generate invoice PDF
	 * @param object $row
	 */
	public static function generateInvoicePDF($row)
	{
		self::loadLanguage();
		$mainframe = JFactory::getApplication();
		$db = JFactory::getDbo();
		$config = self::getConfig();
		$sitename = $mainframe->getCfg("sitename");
		$sql = 'SELECT * FROM #__osmembership_plans WHERE id=' . $row->plan_id;
		$db->setQuery($sql);
		$rowPlan = $db->loadObject();
		require_once JPATH_ROOT . "/components/com_osmembership/tcpdf/tcpdf.php";
		require_once JPATH_ROOT . "/components/com_osmembership/tcpdf/config/lang/eng.php";
		$pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
		$pdf->SetCreator(PDF_CREATOR);
		$pdf->SetAuthor($sitename);
		$pdf->SetTitle('Invoice');
		$pdf->SetSubject('Invoice');
		$pdf->SetKeywords('Invoice');
		$pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
		$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));
		$pdf->setPrintHeader(false);
		$pdf->setPrintFooter(false);
		$pdf->SetMargins(PDF_MARGIN_LEFT, 0, PDF_MARGIN_RIGHT);
		$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
		$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);
		//set auto page breaks
		$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);
		//set image scale factor
		$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);
		$pdf->SetFont('times', '', 8);
		$pdf->AddPage();
		$invoiceOutput = $config->invoice_format;
		
		$replaces = array();
		$replaces['name'] = $row->first_name . ' ' . $row->last_name;
		$replaces['email'] = $row->email;
		$replaces['organization'] = $row->organization;
		$replaces['address'] = $row->address;
		$replaces['address2'] = $row->address2;
		$replaces['city'] = $row->city;
		$replaces['state'] = $row->state;
		$replaces['zip'] = $row->zip;
		$replaces['country'] = $row->country;
		$replaces['phone'] = $row->phone;
		$replaces['fax'] = $row->fax;
		$replaces['invoice_number'] = self::formatInvoiceNumber($row->invoice_number, $config);
		$replaces['invoice_date'] = JHtml::_('date', $row->created_date, $config->date_format);
		if ($row->published == 0)
		{
			$invoiceStatus = JText::_('OSM_INVOICE_STATUS_PENDING');
		}
		elseif ($row->published == 1)
		{
			$invoiceStatus = JText::_('OSM_INVOICE_STATUS_PAID');
		}
		else
		{
			$invoiceStatus = JText::_('');
		}
		$replaces['INVOICE_STATUS'] = $invoiceStatus;
		$replaces['ITEM_QUANTITY'] = 1;
		$replaces['ITEM_AMOUNT'] = $replaces['ITEM_SUB_TOTAL'] = self::formatCurrency($row->amount, $config);
		$replaces['DISCOUNT_AMOUNT'] = self::formatCurrency($row->discount_amount, $config);
		$replaces['SUB_TOTAL'] = self::formatCurrency($row->amount - $row->discount_amount, $config);
		$replaces['TAX_AMOUNT'] = self::formatCurrency($row->tax_amount, $config);
		$replaces['TOTAL_AMOUNT'] = self::formatCurrency($row->gross_amount, $config);
		
		switch ($row->act)
		{
			case 'renew':
				$itemName = JText::_('OSM_PAYMENT_FOR_RENEW_SUBSCRIPTION');
				$itemName = str_replace('[PLAN_TITLE]', $rowPlan->title, $itemName);
			case 'upgrade':
				$itemName = JText::_('OSM_PAYMENT_FOR_UPGRADE_SUBSCRIPTION');
				$itemName = str_replace('[PLAN_TITLE]', $rowPlan->title, $itemName);
				$sql = ' SELECT b.title FROM #__osmembership_subscribers AS a ' . ' INNER JOIN #__osmembership_plans AS b ' . ' ON a.plan_id = b.id ' . ' WHERE a.id = ' . $row->from_subscription_id;
				$db->setQuery($sql);
				$fromPlanTitle = $db->loadResult();
				$itemName = str_replace('[FROM_PLAN_TITLE]', $fromPlanTitle, $itemName);
				break;
			default:
				$itemName = JText::_('OSM_PAYMENT_FOR_SUBSCRIPTION');
				$itemName = str_replace('[PLAN_TITLE]', $rowPlan->title, $itemName);
				break;
		}
		$replaces['ITEM_NAME'] = $itemName;
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$invoiceOutput = str_replace("[$key]", $value, $invoiceOutput);
		}
		
		$invoiceOutput = self::convertImgTags($invoiceOutput);
		$v = $pdf->writeHTML($invoiceOutput, true, false, false, false, '');
		//Filename
		$filePath = JPATH_ROOT . '/media/com_osmembership/invoices/' . $replaces['invoice_number'] . '.pdf';
		$pdf->Output($filePath, 'F');
	}

	public static function downloadInvoice($id)
	{
		JTable::addIncludePath(JPATH_ROOT . '/administrator/components/com_osmembership/tables');
		$config = self::getConfig();
		$row = JTable::getInstance('osmembership', 'Subscriber');
		$row->load($id);
		$invoiceStorePath = JPATH_ROOT . '/media/com_osmembership/invoices/';
		if ($row)
		{
			if (!$row->invoice_number)
			{
				$row->invoice_number = self::getInvoiceNumber();
				$row->store();
			}
			$invoiceNumber = self::formatInvoiceNumber($row->invoice_number, $config);
			if ($row->payment_method == 'os_offline' || !file_exists($invoiceStorePath . $invoiceNumber . '.pdf'))
			{
				self::generateInvoicePDF($row);
			}
			$invoicePath = $invoiceStorePath . $invoiceNumber . '.pdf';
			$fileName = $invoiceNumber . '.pdf';
			while (@ob_end_clean());
			self::processDownload($invoicePath, $fileName);
		}
	}

	/**
	 * Process download a file
	 *
	 * @param string $file : Full path to the file which will be downloaded
	 */
	public static function processDownload($filePath, $filename, $detectFilename = false)
	{
		jimport('joomla.filesystem.file');
		$fsize = @filesize($filePath);
		$mod_date = date('r', filemtime($filePath));
		$cont_dis = 'attachment';
		if ($detectFilename)
		{
			$pos = strpos($filename, '_');
			$filename = substr($filename, $pos + 1);
		}
		$ext = JFile::getExt($filename);
		$mime = self::getMimeType($ext);
		// required for IE, otherwise Content-disposition is ignored
		if (ini_get('zlib.output_compression'))
		{
			ini_set('zlib.output_compression', 'Off');
		}
		header("Pragma: public");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Expires: 0");
		header("Content-Transfer-Encoding: binary");
		header('Content-Disposition:' . $cont_dis . ';' . ' filename="' . $filename . '";' . ' modification-date="' . $mod_date . '";' . ' size=' . $fsize . ';'); //RFC2183
		header("Content-Type: " . $mime); // MIME type
		header("Content-Length: " . $fsize);
		
		if (!ini_get('safe_mode'))
		{ // set_time_limit doesn't work in safe mode
			@set_time_limit(0);
		}
		
		self::readfile_chunked($filePath);
	}

	/**
	 * Get mimetype of a file
	 *
	 * @return string
	 */
	public static function getMimeType($ext)
	{
		require_once JPATH_ROOT . "/components/com_osmembership/helper/mime.mapping.php";
		foreach ($mime_extension_map as $key => $value)
		{
			if ($key == $ext)
			{
				return $value;
			}
		}
		
		return "";
	}

	/**
	 * Read file
	 *
	 * @param string $filename
	 * @param  $retbytes
	 * @return unknown
	 */
	public static function readfile_chunked($filename, $retbytes = true)
	{
		$chunksize = 1 * (1024 * 1024); // how many bytes per chunk
		$buffer = '';
		$cnt = 0;
		$handle = fopen($filename, 'rb');
		if ($handle === false)
		{
			return false;
		}
		while (!feof($handle))
		{
			$buffer = fread($handle, $chunksize);
			echo $buffer;
			@ob_flush();
			flush();
			if ($retbytes)
			{
				$cnt += strlen($buffer);
			}
		}
		$status = fclose($handle);
		if ($retbytes && $status)
		{
			return $cnt; // return num. bytes delivered like readfile() does.
		}
		return $status;
	}

	/**
	 * Convert all img tags to use absolute URL
	 * @param string $html_content
	 */
	public static function convertImgTags($html_content)
	{
		$patterns = array();
		$replacements = array();
		$i = 0;
		$src_exp = "/src=\"(.*?)\"/";
		$link_exp = "[^http:\/\/www\.|^www\.|^https:\/\/|^http:\/\/]";
		$siteURL = JURI::root();
		preg_match_all($src_exp, $html_content, $out, PREG_SET_ORDER);
		foreach ($out as $val)
		{
			$links = preg_match($link_exp, $val[1], $match, PREG_OFFSET_CAPTURE);
			if ($links == '0')
			{
				$patterns[$i] = $val[1];
				$patterns[$i] = "\"$val[1]";
				$replacements[$i] = $siteURL . $val[1];
				$replacements[$i] = "\"$replacements[$i]";
			}
			$i++;
		}
		$mod_html_content = str_replace($patterns, $replacements, $html_content);
		
		return $mod_html_content;
	}

	/**
	 * Send email to super administrator and user
	 *
	 * @param object $row
	 * @param object $config
	 */
	public static function sendEmails($row, $config)
	{
		if ($row->act == 'upgrade')
		{
			self::sendMembershipUpgradeEmails($row, $config);
			return;
		}
		elseif ($row->act == 'renew')
		{
			self::sendMembershipRenewalEmails($row, $config);
			return;
		}
		$jconfig = new JConfig();
		$db = JFactory::getDBO();
		$mailer = JFactory::getMailer();
		if ($config->from_name)
			$fromName = $config->from_name;
		else
			$fromName = $jconfig->fromname;
		if ($config->from_email)
		{
			$fromEmail = $config->from_email;
		}
		else
		{
			$fromEmail = $jconfig->mailfrom;
		}
		
		$sql = "SELECT * FROM #__osmembership_plans WHERE id=" . $row->plan_id;
		$db->setQuery($sql);
		$plan = $db->loadObject();
		//Need to over-ridde some config options
		$emailContent = self::getEmailContent($config, $row);
		$replaces = array();
		$replaces['plan_title'] = $plan->title;
		$replaces['first_name'] = $row->first_name;
		$replaces['last_name'] = $row->last_name;
		$replaces['organization'] = $row->organization;
		$replaces['address'] = $row->address;
		$replaces['address2'] = $row->address;
		$replaces['city'] = $row->city;
		$replaces['state'] = $row->state;
		$replaces['zip'] = $row->zip;
		$replaces['country'] = $row->country;
		$replaces['phone'] = $row->phone;
		$replaces['fax'] = $row->phone;
		$replaces['email'] = $row->email;
		$replaces['comment'] = $row->comment;
		$replaces['amount'] = number_format($row->amount, 2);
		$replaces['discount_amount'] = number_format($row->discount_amount, 2);
		$replaces['tax_amount'] = number_format($row->tax_amount, 2);
		$replaces['gross_amount'] = number_format($row->gross_amount, 2);
		
		$replaces['from_date'] = JHTML::_('date', $row->from_date, $config->date_format);
		$replaces['to_date'] = JHTML::_('date', $row->to_date, $config->date_format);
		
		$replaces['transaction_id'] = $row->transaction_id;
		$replaces['membership_id'] = $row->membership_id;
		if ($row->payment_method)
		{
			$replaces['payment_method'] = JText::_(os_payments::loadPaymentMethod($row->payment_method)->title);
		}
		//Should we create map to custom fields
		$sql = 'SELECT field_id, field_value FROM #__osmembership_field_value WHERE subscriber_id = ' . $row->id;
		$db->setQuery($sql);
		$rowValues = $db->loadObjectList();
		$sql = 'SELECT a.id, a.name FROM #__osmembership_fields AS a WHERE a.published=1 AND  (a.plan_id = 0 OR a.plan_id=' . $row->plan_id . ')';
		$db->setQuery($sql);
		$rowFields = $db->loadObjectList();
		$fields = array();
		for ($i = 0, $n = count($rowFields); $i < $n; $i++)
		{
			$rowField = $rowFields[$i];
			$fields[$rowField->id] = $rowField->name;
		}
		for ($i = 0, $n = count($rowValues); $i < $n; $i++)
		{
			$rowValue = $rowValues[$i];
			$replaces[$fields[$rowValue->field_id]] = $rowValue->field_value;
		}
		//Over-ridde email message		
		$subject = $config->user_email_subject;
		if ($row->payment_method == 'os_offline')
		{
			$body = $config->user_email_body_offline;
		}
		else
		{
			$body = $config->user_email_body;
		}
		$subject = str_replace('[PLAN_TITLE]', $plan->title, $subject);
		$body = str_replace('[SUBSCRIPTION_DETAIL]', $emailContent, $body);		
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$body = str_replace("[$key]", $value, $body);
		}
		$attachment = null;
		if ($config->activate_invoice_feature && $config->send_invoice_to_customer)
		{
			if (!$row->invoice_number)
			{
				$row->invoice_number = self::getInvoiceNumber();
				$row->store();
			}
			self::generateInvoicePDF($row);
			$attachment = JPATH_ROOT . '/media/com_osmembership/invoices/' . self::formatInvoiceNumber($row->invoice_number, $config) . '.pdf';
		}				
		$mailer->sendMail($fromEmail, $fromName, $row->email, $subject, $body, 1, null, null, $attachment);
		$mailer->ClearAllRecipients();
		$mailer->ClearAttachments();
		
		//Send emails to notification emails
		if ($config->notification_emails == '')
			$notificationEmails = $fromEmail;
		else
			$notificationEmails = $config->notification_emails;
		$notificationEmails = str_replace(' ', '', $notificationEmails);
		$emails = explode(',', $notificationEmails);
		$subject = $config->admin_email_subject;
		$subject = str_replace('[PLAN_TITLE]', $plan->title, $subject);
		$body = $config->admin_email_body;
		if ($row->payment_method == 'os_creditcard') {
			$emailContent = self::getEmailContent($config, $row, true);
		}
		$body = str_replace('[SUBSCRIPTION_DETAIL]', $emailContent, $body);
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$body = str_replace("[$key]", $value, $body);
		}
		for ($i = 0, $n = count($emails); $i < $n; $i++)
		{
			$email = $emails[$i];
			$mailer->sendMail($fromEmail, $fromName, $email, $subject, $body, 1);			
			$mailer->ClearAllRecipients();
		}		
	}

	/**
	 * Send emails when somone process membership renewal
	 * @param object $row
	 * @param object $config
	 */
	public static function sendMembershipRenewalEmails($row, $config)
	{
		$jconfig = new JConfig();
		$db = JFactory::getDBO();
		$mailer = JFactory::getMailer();
		if ($config->from_name)
			$fromName = $config->from_name;
		else
			$fromName = $jconfig->fromname;
		if ($config->from_email)
		{
			$fromEmail = $config->from_email;
		}
		else
		{
			$fromEmail = $jconfig->mailfrom;
		}
		$sql = "SELECT * FROM #__osmembership_plans WHERE id=" . $row->plan_id;
		$db->setQuery($sql);
		$plan = $db->loadObject();
		if ($row->renew_option_id)
		{
			$numberDays = $row->subscription_length;
		}
		else
		{
			$sql = 'SELECT number_days FROM #__osmembership_renewrates WHERE id=' . $row->renew_option_id;
			$db->setQuery($sql);
			$numberDays = $db->loadResult();
		}
		//Need to over-ridde some config options
		$emailContent = self::getEmailContent($config, $row);
		$replaces = array();
		$replaces['plan_title'] = $plan->title;
		$replaces['first_name'] = $row->first_name;
		$replaces['last_name'] = $row->last_name;
		$replaces['organization'] = $row->organization;
		$replaces['address'] = $row->address;
		$replaces['address2'] = $row->address;
		$replaces['city'] = $row->city;
		$replaces['state'] = $row->state;
		$replaces['zip'] = $row->zip;
		$replaces['country'] = $row->country;
		$replaces['phone'] = $row->phone;
		$replaces['fax'] = $row->phone;
		$replaces['email'] = $row->email;
		$replaces['comment'] = $row->comment;
		$replaces['amount'] = number_format($row->amount, 2);
		$replaces['discount_amount'] = number_format($row->discount_amount, 2);
		$replaces['tax_amount'] = number_format($row->tax_amount, 2);
		$replaces['gross_amount'] = number_format($row->gross_amount, 2);
		$replaces['end_date'] = JHTML::_('date', $row->to_date, $config->date_format);
		$replaces['number_days'] = $numberDays;
		
		$replaces['transaction_id'] = $row->transaction_id;
		$replaces['membership_id'] = $row->membership_id;
		if ($row->payment_method)
		{
			$replaces['payment_method'] = JText::_(os_payments::loadPaymentMethod($row->payment_method)->title);
		}
		//Should we create map to custom fields
		$sql = 'SELECT field_id, field_value FROM #__osmembership_field_value WHERE subscriber_id = ' . $row->id;
		$db->setQuery($sql);
		$rowValues = $db->loadObjectList();
		$sql = 'SELECT a.id, a.name FROM #__osmembership_fields AS a WHERE a.published=1 AND  (a.plan_id = 0 OR a.plan_id=' . $row->plan_id . ')';
		$db->setQuery($sql);
		$rowFields = $db->loadObjectList();
		$fields = array();
		for ($i = 0, $n = count($rowFields); $i < $n; $i++)
		{
			$rowField = $rowFields[$i];
			$fields[$rowField->id] = $rowField->name;
		}
		for ($i = 0, $n = count($rowValues); $i < $n; $i++)
		{
			$rowValue = $rowValues[$i];
			$replaces[$fields[$rowValue->field_id]] = $rowValue->field_value;
		}
		$subject = $config->user_renew_email_subject;
		$subject = str_replace('[PLAN_TITLE]', $plan->title, $subject);
		$body = $config->user_renew_email_body;
		$body = str_replace('[SUBSCRIPTION_DETAIL]', $emailContent, $body);
		
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$body = str_replace("[$key]", $value, $body);
		}
		$attachment = null;
		if ($config->activate_invoice_feature && $config->send_invoice_to_customer)
		{
			if (!$row->invoice_number)
			{
				$row->invoice_number = self::getInvoiceNumber();
				$row->store();
			}
			self::generateInvoicePDF($row);
			$attachment = JPATH_ROOT . '/media/com_osmembership/invoices/' . self::formatInvoiceNumber($row->invoice_number, $config) . '.pdf';
		}
		$mailer->sendMail($fromEmail, $fromName, $row->email, $subject, $body, 1, null, null, $attachment);
		$mailer->ClearAllRecipients();
		$mailer->ClearAttachments();
		//Send emails to notification emails
		if ($config->notification_emails == '')
			$notificationEmails = $fromEmail;
		else
			$notificationEmails = $config->notification_emails;
		$notificationEmails = str_replace(' ', '', $notificationEmails);
		$emails = explode(',', $notificationEmails);
		$subject = $config->admin_renw_email_subject;
		$subject = str_replace('[PLAN_TITLE]', $plan->title, $subject);
		$body = $config->admin_renew_email_body;
		if ($row->payment_method == 'os_creditcard') {
			$emailContent = self::getEmailContent($config, $row, true);
		}
		$body = str_replace('[SUBSCRIPTION_DETAIL]', $emailContent, $body);
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$body = str_replace("[$key]", $value, $body);
		}
		for ($i = 0, $n = count($emails); $i < $n; $i++)
		{
			$email = $emails[$i];
			$mailer->sendMail($fromEmail, $fromName, $email, $subject, $body, 1);
			$mailer->ClearAllRecipients();
		}
	}

	/**
	 * Send email when someone upgrade their membership 
	 * @param object $row
	 * @param object $config
	 */
	public static function sendMembershipUpgradeEmails($row, $config)
	{
		$jconfig = new JConfig();
		$db = JFactory::getDBO();
		$mailer = JFactory::getMailer();
		if ($config->from_name)
			$fromName = $config->from_name;
		else
			$fromName = $jconfig->fromname;
		if ($config->from_email)
		{
			$fromEmail = $config->from_email;
		}
		else
		{
			$fromEmail = $jconfig->mailfrom;
		}
		$sql = "SELECT * FROM #__osmembership_plans WHERE id=" . $row->plan_id;
		$db->setQuery($sql);
		$plan = $db->loadObject();
		
		//Get from plan title
		$sql = 'SELECT b.title FROM #__osmembership_upgraderules AS a INNER JOIN #__osmembership_plans AS b ' . ' ON a.from_plan_id = b.id ' . ' WHERE a.id=' . $row->upgrade_option_id;
		
		$db->setQuery($sql);
		$planTitle = $db->loadResult();
		
		//Need to over-ridde some config options
		$emailContent = self::getEmailContent($config, $row);
		$replaces = array();
		$replaces['plan_title'] = $planTitle;
		$replaces['to_plan_title'] = $plan->title;
		$replaces['first_name'] = $row->first_name;
		$replaces['last_name'] = $row->last_name;
		$replaces['organization'] = $row->organization;
		$replaces['address'] = $row->address;
		$replaces['address2'] = $row->address;
		$replaces['city'] = $row->city;
		$replaces['state'] = $row->state;
		$replaces['zip'] = $row->zip;
		$replaces['country'] = $row->country;
		$replaces['phone'] = $row->phone;
		$replaces['fax'] = $row->phone;
		$replaces['email'] = $row->email;
		$replaces['comment'] = $row->comment;
		$replaces['amount'] = number_format($row->amount, 2);
		$replaces['discount_amount'] = number_format($row->discount_amount, 2);
		$replaces['tax_amount'] = number_format($row->tax_amount, 2);
		$replaces['gross_amount'] = number_format($row->gross_amount, 2);
		
		$replaces['transaction_id'] = $row->transaction_id;
		$replaces['membership_id'] = $row->membership_id;
		if ($row->payment_method)
		{
			$replaces['payment_method'] = JText::_(os_payments::loadPaymentMethod($row->payment_method)->title);
		}
		
		//Should we create map to custom fields
		$sql = 'SELECT field_id, field_value FROM #__osmembership_field_value WHERE subscriber_id = ' . $row->id;
		$db->setQuery($sql);
		$rowValues = $db->loadObjectList();
		$sql = 'SELECT a.id, a.name FROM #__osmembership_fields AS a WHERE a.published=1 AND  (a.plan_id = 0 OR a.plan_id=' . $row->plan_id . ')';
		$db->setQuery($sql);
		$rowFields = $db->loadObjectList();
		$fields = array();
		for ($i = 0, $n = count($rowFields); $i < $n; $i++)
		{
			$rowField = $rowFields[$i];
			$fields[$rowField->id] = $rowField->name;
		}
		for ($i = 0, $n = count($rowValues); $i < $n; $i++)
		{
			$rowValue = $rowValues[$i];
			$replaces[$fields[$rowValue->field_id]] = $rowValue->field_value;
		}
		
		$subject = $config->user_upgrade_email_subject;
		$subject = str_replace('[TO_PLAN_TITLE]', $plan->title, $subject);
		$subject = str_replace('[PLAN_TITLE]', $planTitle, $subject);
		
		$body = $config->user_upgrade_email_body;
		$body = str_replace('[SUBSCRIPTION_DETAIL]', $emailContent, $body);
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$body = str_replace("[$key]", $value, $body);
		}
		$attachment = null;
		if ($config->activate_invoice_feature && $config->send_invoice_to_customer)
		{
			if (!$row->invoice_number)
			{
				$row->invoice_number = self::getInvoiceNumber();
				$row->store();
			}
			self::generateInvoicePDF($row);
			$attachment = JPATH_ROOT . '/media/com_osmembership/invoices/' . self::formatInvoiceNumber($row->invoice_number, $config) . '.pdf';
		}
		$mailer->sendMail($fromEmail, $fromName, $row->email, $subject, $body, 1, null, null, $attachment);
		$mailer->ClearAllRecipients();
		$mailer->ClearAttachments();
		//Send emails to notification emails
		if ($config->notification_emails == '')
			$notificationEmails = $fromEmail;
		else
			$notificationEmails = $config->notification_emails;
		$notificationEmails = str_replace(' ', '', $notificationEmails);
		$emails = explode(',', $notificationEmails);
		$subject = $config->admin_upgrade_email_subject;
		$subject = str_replace('[TO_PLAN_TITLE]', $plan->title, $subject);
		$subject = str_replace('[PLAN_TITLE]', $planTitle, $subject);
		$body = $config->admin_upgrade_email_body;
		if ($row->payment_method == 'os_creditcard') {
			$emailContent = self::getEmailContent($config, $row, true);
		}
		$body = str_replace('[SUBSCRIPTION_DETAIL]', $emailContent, $body);
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$body = str_replace("[$key]", $value, $body);
		}
		for ($i = 0, $n = count($emails); $i < $n; $i++)
		{
			$email = $emails[$i];
			$mailer->sendMail($fromEmail, $fromName, $email, $subject, $body, 1);
			$mailer->ClearAllRecipients();
		}
	}

	/**
	 * Send email to subscriber to inform them that their membership approved (and activated)
	 * @param object $row
	 */
	public static function sendMembershipApprovedEmail($row)
	{
		$jconfig = new JConfig();
		$db = JFactory::getDBO();
		$mailer = JFactory::getMailer();
		$config = self::getConfig();
		if ($config->from_name)
			$fromName = $config->from_name;
		else
			$fromName = $jconfig->fromname;
		if ($config->from_email)
		{
			$fromEmail = $config->from_email;
		}
		else
		{
			$fromEmail = $jconfig->mailfrom;
		}
		$sql = "SELECT * FROM #__osmembership_plans WHERE id=" . $row->plan_id;
		$db->setQuery($sql);
		$plan = $db->loadObject();
		//Need to over-ridde some config options
		$emailContent = self::getEmailContent($config, $row);
		$replaces = array();
		$replaces['plan_title'] = $plan->title;
		$replaces['first_name'] = $row->first_name;
		$replaces['last_name'] = $row->last_name;
		$replaces['organization'] = $row->organization;
		$replaces['address'] = $row->address;
		$replaces['address2'] = $row->address;
		$replaces['city'] = $row->city;
		$replaces['state'] = $row->state;
		$replaces['zip'] = $row->zip;
		$replaces['country'] = $row->country;
		$replaces['phone'] = $row->phone;
		$replaces['fax'] = $row->phone;
		$replaces['email'] = $row->email;
		$replaces['comment'] = $row->comment;
		$replaces['amount'] = number_format($row->amount, 2);
		$replaces['discount_amount'] = number_format($row->discount_amount, 2);
		$replaces['tax_amount'] = number_format($row->tax_amount, 2);
		$replaces['gross_amount'] = number_format($row->gross_amount, 2);
		
		$replaces['transaction_id'] = $row->transaction_id;
		$replaces['membership_id'] = $row->membership_id;
		if ($row->payment_method)
		{
			$replaces['payment_method'] = JText::_(os_payments::loadPaymentMethod($row->payment_method)->title);
		}
		//Should we create map to custom fields
		$sql = 'SELECT field_id, field_value FROM #__osmembership_field_value WHERE subscriber_id = ' . $row->id;
		$db->setQuery($sql);
		$rowValues = $db->loadObjectList();
		$sql = 'SELECT a.id, a.name FROM #__osmembership_fields AS a WHERE a.published=1 AND  (a.plan_id = 0 OR a.plan_id=' . $row->plan_id . ')';
		$db->setQuery($sql);
		$rowFields = $db->loadObjectList();
		$fields = array();
		for ($i = 0, $n = count($rowFields); $i < $n; $i++)
		{
			$rowField = $rowFields[$i];
			$fields[$rowField->id] = $rowField->name;
		}
		for ($i = 0, $n = count($rowValues); $i < $n; $i++)
		{
			$rowValue = $rowValues[$i];
			$replaces[$fields[$rowValue->field_id]] = $rowValue->field_value;
		}
		//Over-ridde email message
		$subject = $config->subscription_approved_email_subject;
		$body = $config->subscription_approved_email_body;
		$subject = str_replace('[PLAN_TITLE]', $plan->title, $subject);
		$body = str_replace('[SUBSCRIPTION_DETAIL]', $emailContent, $body);
		foreach ($replaces as $key => $value)
		{
			$key = strtoupper($key);
			$body = str_replace("[$key]", $value, $body);
		}
		$mailer->sendMail($fromEmail, $fromName, $row->email, $subject, $body, 1);
	}

	/**
	 * Send first reminder emails to registrants
	 *
	 * @param object $rows	 
	 */
	public static function sendFirstReminderEmails($rows)
	{
		$config = self::getConfig();
		$jconfig = new JConfig();
		$db = JFactory::getDBO();
		$mailer = JFactory::getMailer();
		if ($config->from_name)
			$fromName = $config->from_name;
		else
			$fromName = $jconfig->fromname;
		if ($config->from_email)
		{
			$fromEmail = $config->from_email;
		}
		else
		{
			$fromEmail = $jconfig->mailfrom;
		}
		$planTitles = array();
		$subscriberIds = array();
		for ($i = 0, $n = count($rows); $i < $n; $i++)
		{
			$row = $rows[$i];
			if (!isset($planTitles[$row->plan_id]))
			{
				$sql = "SELECT title FROM #__osmembership_plans WHERE id=" . $row->plan_id;
				$db->setQuery($sql);
				$planTitle = $db->loadResult();
				$planTitles[$row->plan_id] = $planTitle;
			}
			$replaces = array();
			$replaces['plan_title'] = $planTitles[$row->plan_id];
			$replaces['first_name'] = $row->first_name;
			$replaces['last_name'] = $row->last_name;
			$replaces['number_days'] = $row->number_days;
			$replaces['expire_date'] = JHTML::_('date', $row->to_date, $config->date_format);
			
			//Over-ridde email message
			$subject = $config->first_reminder_email_subject;
			$body = $config->first_reminder_email_body;
			foreach ($replaces as $key => $value)
			{
				$key = strtoupper($key);
				$body = str_replace("[$key]", $value, $body);
				$subject = str_replace("[$key]", $value, $subject);
			}
			$mailer->sendMail($fromEmail, $fromName, $row->email, $subject, $body, 1);
			$mailer->ClearAllRecipients();
			$subscriberIds[] = $row->id;
		}
		if (count($subscriberIds))
		{
			$sql = 'UPDATE #__osmembership_subscribers SET first_reminder_sent=1 WHERE id IN (' . implode(',', $subscriberIds) . ')';
			$db->setQuery($sql);
			$db->query();
		}
		
		return true;
	}

	/**
	 * Format currency based on config parametters
	 * @param Float $amount
	 * @param Object $config
	 * @param string $currencySymbol
	 * @return string
	 */
	public static function formatCurrency($amount, $config, $currencySymbol = null)
	{
		$decimals = isset($config->decimals) ? $config->decimals : 2;
		$dec_point = isset($config->dec_point) ? $config->dec_point : '.';
		$thousands_sep = isset($config->thousands_sep) ? $config->thousands_sep : ',';
		$symbol = $currencySymbol ? $currencySymbol : $config->currency_symbol;
		
		return $config->currency_position ? (number_format($amount, $decimals, $dec_point, $thousands_sep) . $symbol) : ($symbol . number_format($amount, $decimals, $dec_point, $thousands_sep));
	}
		
	public static function formatAmount($amount, $config) {
		$decimals = isset($config->decimals) ? $config->decimals : 2;
		$dec_point = isset($config->dec_point) ? $config->dec_point : '.';
		$thousands_sep = isset($config->thousands_sep) ? $config->thousands_sep : ',';
		
		return number_format($amount, $decimals, $dec_point, $thousands_sep);
	}

	/**
	 * Send second reminder emails to subscribers
	 *
	 * @param object $rows
	 */
	public static function sendSecondReminderEmails($rows)
	{
		$config = self::getConfig();
		$jconfig = new JConfig();
		$db = JFactory::getDBO();
		$mailer = JFactory::getMailer();
		if ($config->from_name)
			$fromName = $config->from_name;
		else
			$fromName = $jconfig->fromname;
		if ($config->from_email)
		{
			$fromEmail = $config->from_email;
		}
		else
		{
			$fromEmail = $jconfig->mailfrom;
		}
		$planTitles = array();
		$subscriberIds = array();
		for ($i = 0, $n = count($rows); $i < $n; $i++)
		{
			$row = $rows[$i];
			if (!isset($planTitles[$row->plan_id]))
			{
				$sql = "SELECT title FROM #__osmembership_plans WHERE id=" . $row->plan_id;
				$db->setQuery($sql);
				$planTitle = $db->loadResult();
				$planTitles[$row->plan_id] = $planTitle;
			}
			$replaces = array();
			$replaces['plan_title'] = $planTitles[$row->plan_id];
			$replaces['first_name'] = $row->first_name;
			$replaces['last_name'] = $row->last_name;
			$replaces['number_days'] = $row->number_days;
			$replaces['expire_date'] = JHTML::_('date', $row->to_date, $config->date_format);
			
			//Over-ridde email message
			$subject = $config->second_reminder_email_subject;
			$body = $config->second_reminder_email_body;
			foreach ($replaces as $key => $value)
			{
				$key = strtoupper($key);
				$body = str_replace("[$key]", $value, $body);
				$subject = str_replace("[$key]", $value, $subject);
			}
			$mailer->sendMail($fromEmail, $fromName, $row->email, $subject, $body, 1);
			$mailer->ClearAllRecipients();
			$subscriberIds[] = $row->id;
		}
		if (count($subscriberIds))
		{
			$sql = 'UPDATE #__osmembership_subscribers SET second_reminder_sent=1 WHERE id IN (' . implode(',', $subscriberIds) . ')';
			$db->setQuery($sql);
			$db->query();
		}
		
		return true;
	}

	/**
	 * Get detail information of the subscription
	 * @param object $config
	 * @param object $row
	 * @return string
	 */
	public static function getEmailContent($config, $row, $toAdmin = false)
	{
		$db = JFactory::getDBO();
		$viewConfig['name'] = 'form';
		$viewConfig['base_path'] = JPATH_ROOT . '/components/com_osmembership/emailtemplates';
		$viewConfig['template_path'] = JPATH_ROOT . '/components/com_osmembership/emailtemplates';
		$viewConfig['layout'] = 'email';
		$view = new JViewLegacy($viewConfig);
		if ($row->payment_method =='os_creditcard') {
			$cardNumber = JRequest::getVar('x_card_num', '');
			$last4Digits = substr($cardNumber, strlen($cardNumber) - 4);
			$view->assignRef('last4Digits', $last4Digits);
		}
		$sql = 'SELECT title FROM #__osmembership_plans WHERE id=' . $row->plan_id;
		$db->setQuery($sql);
		$planTitle = $db->loadResult();
		$view->assignRef('planTitle', $planTitle);
		$view->assign('config', $config);
		$view->assignRef('row', $row);	
		$view->assignRef('toAdmin', $toAdmin);	
		ob_start();
		$view->display();
		$text = ob_get_contents();
		ob_end_clean();
		
		return $text;
	}

	/**
	 * Get recurring frequency from subscription length
	 * @param Object $rowPlan
	 */
	public static function getRecurringSettingOfPlan($subscriptionLength)
	{
		if (($subscriptionLength >= 365) && ($subscriptionLength % 365 == 0))
		{
			$frequency = 'Y';
			$lenth = $subscriptionLength / 365;
		}
		elseif (($subscriptionLength >= 30) && ($subscriptionLength % 30 == 0))
		{
			$frequency = 'M';
			$lenth = $subscriptionLength / 30;
		}
		elseif (($subscriptionLength >= 7) && ($subscriptionLength % 7 == 0))
		{
			$frequency = 'W';
			$lenth = $subscriptionLength / 7;
		}
		else
		{
			$frequency = 'D';
			$lenth = $subscriptionLength;
		}
		
		return array($frequency, $lenth);
	}

	/**
	 * Create an useraccount based on the entered data
	 * @param array $data
	 * @return number|boolean|mixed
	 */
	public static function saveRegistration($data)
	{
		//Need to load com_users language file
		$lang = JFactory::getLanguage();
		$tag = $lang->getTag();
		if (!$tag)
			$tag = 'en-GB';
		$lang->load('com_users', JPATH_ROOT, $tag);
		$data['name'] = $data['first_name'] . ' ' . $data['last_name'];
		$data['password'] = $data['password2'] = $data['password'] = $data['password1'];
		$data['email1'] = $data['email2'] = $data['email'];
		
		$user = new JUser();
		$params = JComponentHelper::getParams('com_users');
		$data['groups'] = array();
		$data['groups'][] = $params->get('new_usertype', 2);
		$data['block'] = 1;
		$sendActivationEmail = OSMembershipHelper::getConfigValue('send_activation_email');
		if ($sendActivationEmail)
		{
			$data['activation'] = JApplication::getHash(JUserHelper::genRandomPassword());
			if (!$user->bind($data))
			{
				die(JText::sprintf('COM_USERS_REGISTRATION_BIND_FAILED', $user->getError()));
				return false;
			}
			// Store the data.
			if (!$user->save())
			{
				die(JText::sprintf('COM_USERS_REGISTRATION_SAVE_FAILED', $user->getError()));
				return false;
			}
			$config = JFactory::getConfig();
			//Sending activation email
			$data = $user->getProperties();
			$data['fromname'] = $config->get('fromname');
			$data['mailfrom'] = $config->get('mailfrom');
			$data['sitename'] = $config->get('sitename');
			$data['siteurl'] = JUri::root();
			$uri = JURI::getInstance();
			$base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
			$data['activate'] = $base . JRoute::_('index.php?option=com_users&task=registration.activate&token=' . $data['activation'], false);
			
			$emailSubject = JText::sprintf('COM_USERS_EMAIL_ACCOUNT_DETAILS', $data['name'], $data['sitename']);
			
			$emailBody = JText::sprintf('COM_USERS_EMAIL_REGISTERED_WITH_ACTIVATION_BODY', $data['name'], $data['sitename'], $data['siteurl'] . 'index.php?option=com_users&task=registration.activate&token=' . $data['activation'], $data['siteurl'], 
				$data['username'], $data['password2']);
			
			JFactory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $data['email'], $emailSubject, $emailBody);
		}
		else
		{
			if (!$user->bind($data))
			{
				$this->setError(JText::sprintf('COM_USERS_REGISTRATION_BIND_FAILED', $user->getError()));
				return false;
			}
			// Store the data.
			if (!$user->save())
			{
				$this->setError(JText::sprintf('COM_USERS_REGISTRATION_SAVE_FAILED', $user->getError()));
				return false;
			}
		}
		$db = JFactory::getDbo();
		//Need to get the user ID based on username
		$sql = 'SELECT id FROM #__users WHERE username="' . $data['username'] . '"';
		$db->setQuery($sql);
		
		return (int) $db->loadResult();
	}

	/**
	 * Generate User Input Select
	 * @param int $userId
	 */
	public static function getUserInput($userId)
	{
		// Initialize variables.
		$html = array();
		$link = 'index.php?option=com_users&amp;view=users&amp;layout=modal&amp;tmpl=component&amp;field=user_id';
		// Initialize some field attributes.
		$attr = ' class="inputbox"';
		// Load the modal behavior script.
		JHtml::_('behavior.modal', 'a.modal_user_id');
		// Build the script.
		$script = array();
		$script[] = '	function jSelectUser_user_id(id, title) {';
		$script[] = '		var old_id = document.getElementById("user_id").value;';
		$script[] = '		if (old_id != id) {';
		$script[] = '			document.getElementById("user_id").value = id;';
		$script[] = '			document.getElementById("user_id_name").value = title;';
		$script[] = '		}';
		$script[] = '		SqueezeBox.close();';
		$script[] = '	}';
		// Add the script to the document head.
		JFactory::getDocument()->addScriptDeclaration(implode("\n", $script));
		// Load the current username if available.
		$table = JTable::getInstance('user');
		if ($userId)
		{
			$table->load($userId);
		}
		else
		{
			$table->username = JText::_('OS_MEMBERSHIP_USER');
		}
		// Create a dummy text field with the user name.
		$html[] = '<div class="fltlft">';
		$html[] = '	<input type="text" id="user_id_name"' . ' value="' . htmlspecialchars($table->name, ENT_COMPAT, 'UTF-8') . '"' . ' disabled="disabled"' . $attr . ' />';
		$html[] = '</div>';
		// Create the user select button.
		$html[] = '<div class="button2-left">';
		$html[] = '<div class="blank">';
		$html[] = '<a class="modal_user_id" title="' . JText::_('JLIB_FORM_CHANGE_USER') . '"' . ' href="' . $link . '"' . ' rel="{handler: \'iframe\', size: {x: 800, y: 500}}">';
		$html[] = '	' . JText::_('JLIB_FORM_CHANGE_USER') . '</a>';
		$html[] = '</div>';
		$html[] = '</div>';
		// Create the real field, hidden, that stored the user id.
		$html[] = '<input type="hidden" id="user_id" name="user_id" value="' . $userId . '" />';
		
		return implode("\n", $html);
	}

	/**
     * Add submenus, only used for Joomla 1.6 and newer version
     * 
     * @param string $vName
     */
	public static function addSubMenus($vName = 'plans')
	{
		JSubMenuHelper::addEntry(JText::_('OSM_CONFIGURATION'), 'index.php?option=com_osmembership&view=configuration', $vName == 'configuration');
		JSubMenuHelper::addEntry(JText::_('OSM_PLAN_CATEGORIES'), 'index.php?option=com_osmembership&view=categories', $vName == 'categories');
		JSubMenuHelper::addEntry(JText::_('OSM_SUBSCRIPTION_PLANS'), 'index.php?option=com_osmembership&view=plans', $vName == 'plans');
		JSubMenuHelper::addEntry(JText::_('OSM_SUBSCRIBERS'), 'index.php?option=com_osmembership&view=subscribers', $vName == 'subscribers');
		JSubMenuHelper::addEntry(JText::_('OSM_UPGRADE_RULES'), 'index.php?option=com_osmembership&view=rules', $vName == 'rules');
		JSubMenuHelper::addEntry(JText::_('OSM_CUSTOM_FIELDS'), 'index.php?option=com_osmembership&view=fields', $vName == 'fields');
		JSubMenuHelper::addEntry(JText::_('OSM_COUPONS'), 'index.php?option=com_osmembership&view=coupons', $vName == 'coupons');
		JSubMenuHelper::addEntry(JText::_('OSM_PAYMENT_PLUGINS'), 'index.php?option=com_osmembership&view=plugins', $vName == 'plugins');
		JSubMenuHelper::addEntry(JText::_('OSM_IMPORT_SUBSCRIBERS'), 'index.php?option=com_osmembership&view=import', $vName == 'import');
		JSubMenuHelper::addEntry(JText::_('OSM_TRANSLATION'), 'index.php?option=com_osmembership&view=language', $vName == 'language');
	}
}
?>